using System;
using NUnit.Framework;

namespace RFSExplorer {
	public enum RFSObjectType {
		File,
		Directory
	}

	/// <summary>
	/// Summary description for RFSObject.
	/// </summary>
	public class RFSObject : IComparable {
		#region Properties that effect the descriptor.
		private int partitionNumber;
		private int driveNumber;
		private string absolutePath;
		public int PartitionNumber {
			get { return partitionNumber; }
			set { partitionNumber = value; UpdateDescriptor(); }
		}
		public int DriveNumber {
			get { return driveNumber; }
			set { driveNumber = value; UpdateDescriptor(); }
		}
		public string AbsolutePath {
			get { return absolutePath; }
			set { absolutePath = value; UpdateDescriptor(); }
		}
		#endregion

		private string descriptor;
		/// <summary>
		/// Returns the descriptor ("DriveNumber.PartitionNumber AbsolutePath") for this object.
		/// </summary>
		public string Descriptor {
			get { return descriptor; }
		}
		private void UpdateDescriptor() {
			descriptor = string.Format("{0}.{1} \"{2}\"", driveNumber, partitionNumber, absolutePath);
		}

		public RFSObject ContainingDirectory;
		public string Name;
		public string Permissions;
		public string Label;
		public string DateModified;
		public double Size;
		public RFSObjectType ObjectType = RFSObjectType.Directory;

		public RFSObject(RFSObjectType objectType, int driveNumber, int partitionNumber, string absolutePath) {
			ObjectType = objectType;
			DriveNumber = driveNumber;
			PartitionNumber = partitionNumber;
			AbsolutePath = absolutePath;
		}

		/// <summary>
		/// This will probably be used for just creating a "ROOT" (/) node so we won't have a containing directory.
		/// </summary>
		public RFSObject(RFSObjectType objectType, int driveNumber, int partitionNumber, string name, string label, string absolutePath) {
			ObjectType = objectType;
			Name = name;
			Label = label;

			this.driveNumber = driveNumber;
			this.partitionNumber = partitionNumber;
			this.absolutePath = absolutePath;
			UpdateDescriptor();
		}

		/// <summary>
		/// This should be used for creating ANY nodes underneath the root node, therefore some parameters can be initialized
		/// from the containingDirectory parameter.
		/// In case of objectType = File, the Size should be set separately.
		/// </summary>
		public RFSObject(RFSObjectType objectType, RFSObject containingDirectory, string name, string label, string dateModified, string permissions) {
			if (containingDirectory == null) {
				throw new NullReferenceException("Containing directory cannot be null");
			}
			ObjectType = objectType;
			ContainingDirectory = containingDirectory;
			Name = name;
			Label = label;
			DateModified = dateModified;
			Permissions = permissions;

			string contAbsPath = containingDirectory.absolutePath;
			this.absolutePath = contAbsPath +  (contAbsPath.Substring(contAbsPath.Length-1, 1) != "/" ? "/" : "") + name;
			this.partitionNumber = containingDirectory.PartitionNumber;
			this.driveNumber = containingDirectory.DriveNumber;
			UpdateDescriptor();
		}

		public string GetHumanReadbleSize() {
			if (ObjectType == RFSObjectType.Directory) { return ""; }
			return Size + " b";
		}


		#region IComparable Members

		/// <summary>
		/// Compares this object to another RFSObject. A directory is always "less than" a file.
		/// </summary>
		/// <exception cref="ArgumentException">Throws an argument exception if obj is not of type RFSObject.</exception>
		/// <returns></returns>
		public int CompareTo(object obj) {
			if (obj == null) return 1; // If the object is null then assume this object is greater.
			if (obj.GetType() == typeof(string)) { return this.Name.CompareTo(obj); }

			if (obj.GetType() != typeof(RFSObject)) {
				throw new ArgumentException("Only comparison with RFSObject is supported.");
			}

			RFSObject other = (RFSObject)obj;

			// If we do not have the same object types then the "Directory" wins (File=0 and Directory=1).
			if (other.ObjectType != this.ObjectType) { return (this.ObjectType == RFSObjectType.Directory ? -1 : 1); }

			// Otherwise we have to string compare the names.
			return this.Name.CompareTo(other.Name);
		}

		#endregion
	}

	[TestFixture]
	public class RFSObjectTests {
		[Test]
		public void IComparable_Tests() {
			RFSObject dirA = new RFSObject(RFSObjectType.Directory, 1, 2, "ADir1","ADir1","/ADir1");
			RFSObject dirB = new RFSObject(RFSObjectType.Directory, 1, 2, "BDir2","BDir2","/BDir2");
			RFSObject fileA = new RFSObject(RFSObjectType.File, 1, 2, "File1", "File1","/File1");
			RFSObject fileB = new RFSObject(RFSObjectType.File, 1, 2, "File2", "File2","/File2");
			RFSObject fileC = new RFSObject(RFSObjectType.File, 1, 2, "File2", "File2","/File2");

			// dirA = dirA
			Assert.AreEqual(dirA.CompareTo(dirA), 0);   // Directory
			Assert.AreEqual(fileA.CompareTo(fileA), 0); // File
			
			Assert.IsTrue(dirA.CompareTo(dirB)<0);    // "Dir1" < "Dir2"

			Assert.IsTrue(fileA.CompareTo(fileB)<0);  // "File1" < "File2"
			Assert.IsTrue(fileB.CompareTo(fileA)>0);  // "File2" > "File2"
			Assert.IsTrue(fileB.CompareTo(fileC)==0); // "File2" == "File2"

			Assert.IsTrue(dirA.CompareTo(fileA)<0);   // "Dir1" < "File1"
			Assert.IsTrue(fileB.CompareTo(dirB)>0);   // "File2" > "Dir2"
		}

		[Test]
		public void IComparable_Equal_Test() {
			RFSObject dirA = new RFSObject(RFSObjectType.Directory, 1, 2, "Dir1","Dir1","/Dir1");
			RFSObject dirB = new RFSObject(RFSObjectType.Directory, 1, 2, "Dir1","Dir1","/Dir1");
			RFSObject dirC = new RFSObject(RFSObjectType.Directory, 1, 2, "Dir1","Dir1","/Dir1");

			// A=B then B=A
			Assert.AreEqual(dirA.CompareTo(dirB), dirB.CompareTo(dirA));

			// A=B & B=C then C=A  (Note, A=B was checked above so we only need to check B=A and C=A.
			Assert.AreEqual(dirB.CompareTo(dirC), dirC.CompareTo(dirA));
		}

		[Test]
		[ExpectedException(typeof(ArgumentException))]
		public void IComparableException_Test() {
			RFSObject a = new RFSObject(RFSObjectType.File, 1, 1, "/");
			a.CompareTo("A String");
		}
	}
}