using System;
using System.Diagnostics;
using System.IO;
using System.Xml;

namespace RFSExplorer {
	/// <summary>
	/// NOTE: Directories should NOT have the ending "\".
	/// </summary>
	public class Settings {
		#region Static variables and Functions
		private static Settings onlyInstance = null;

		public static Settings GetInstance() {
			if (onlyInstance == null) { onlyInstance = new Settings(); }
			return onlyInstance;
		}
		#endregion

		private string configFilename = ""; 
		private string rfstoolPath = "";
		private ProcessStartInfo rfsStartInfo = null;

		#region Public Properties
		public bool ExpandNodesOnSelect = false;
		public string TemporaryDirectory = System.IO.Path.GetTempPath() + "RFSExplorer";
		
		#region Main form settings
		// Tree View Settings
		public int MainForm_TreeView_Width = 200;

		// List View Settings
		public int MainForm_ListView_LastMod_Width = 50;
		public int MainForm_ListView_Perm_Width = 50;
		public int MainForm_ListView_Name_Width = 50;
		public int MainForm_ListView_Size_Width = 50;
		public string MainForm_ListView_View = "LargeIcon";
		#endregion
		#endregion

		public string RFSToolPath {
			get { return rfstoolPath; }
			set { rfstoolPath = value; }
		}

		public string RFSToolExe {
			get { return rfstoolPath + "\\rfstool.exe"; }
		}

		private Settings() {}

		public void SetConfigFile(string filePath) {
			configFilename = filePath;
		}
		
		/// <summary>
		/// Makes sure that the current settings valid.
		/// </summary>
		/// <param name="error">This will contain an error string.</param>
		/// <returns></returns>
		public bool AreRequiredSettingsValid(out string error) {
			error = "";
			if (!File.Exists(RFSToolExe)) {
				error = "The RFS tool exe was not found, please fix by going to Tools -> Settings.";
				return false;
			}
			if (!Directory.Exists(TemporaryDirectory)) {
				if (System.Windows.Forms.MessageBox.Show("The temporary directory (" + TemporaryDirectory + 
					") was not found, should I create it?", "Temporary Directory Not Found",
					System.Windows.Forms.MessageBoxButtons.YesNo) == System.Windows.Forms.DialogResult.Yes) {
					try {
						Directory.CreateDirectory(TemporaryDirectory);
					} catch (Exception ex) {
						error = "Unable to create temporary directory (" + TemporaryDirectory + ").\n" + ex.Message;
						return false;
					}
				} else {
					error = "The temporary directory does not exist, please fix by going to Tools -> Settings.";
					return false;
				}
			}
			return true;
		}

		public ProcessStartInfo GetRFSToolStartInfo() {
			if (rfsStartInfo == null) {
				rfsStartInfo = new ProcessStartInfo(this.RFSToolExe);
				rfsStartInfo.RedirectStandardOutput = true;
				rfsStartInfo.CreateNoWindow = true;
				rfsStartInfo.UseShellExecute = false;
			} else if (rfsStartInfo.FileName != this.RFSToolExe) {
				rfsStartInfo.FileName = this.RFSToolExe;
			}

			return rfsStartInfo;
		}

		public void LoadSettings(string fileName) {
			configFilename = fileName;
			if (!File.Exists(fileName)) { return; }

			XmlDocument xmlDoc = new XmlDocument();
			try {
				xmlDoc.Load(configFilename);
				RFSToolPath = GetNodeText(xmlDoc, "/Config/RFSPath");
				TemporaryDirectory = GetNodeText(xmlDoc, "/Config/TemporaryDirectory");
				MainForm_TreeView_Width = int.Parse(GetNodeText(xmlDoc, "/Config/MainForm/TreeWidth"));
				MainForm_ListView_LastMod_Width = int.Parse(GetNodeText(xmlDoc, "/Config/MainForm/ListView/Column_LastModified_Width"));
				MainForm_ListView_Perm_Width = int.Parse(GetNodeText(xmlDoc, "/Config/MainForm/ListView/Column_Permissions_Width"));
				MainForm_ListView_Name_Width = int.Parse(GetNodeText(xmlDoc, "/Config/MainForm/ListView/Column_Name_Width"));
				MainForm_ListView_Size_Width = int.Parse(GetNodeText(xmlDoc, "/Config/MainForm/ListView/Column_Size_Width"));
				MainForm_ListView_View = GetNodeText(xmlDoc, "/Config/MainForm/ListView/Column_View_Width");
			} catch (Exception ex) {
				System.Windows.Forms.MessageBox.Show(ex.ToString());
			}
		}

		private string GetNodeText(XmlDocument xmlDoc, string XPath) {
			XmlNode node = xmlDoc.SelectSingleNode(XPath);
			if (node == null) { return ""; }
			foreach (XmlNode n in node.ChildNodes) {
				if (n.NodeType == XmlNodeType.Text) {
					return n.Value;
				}
			}
			return "";
		}

		public void WriteSettings() {
			try {
				XmlDocument xmlDoc = new XmlDocument();
				xmlDoc.CreateXmlDeclaration("1.0", null, null);
				XmlElement rootConfig = xmlDoc.CreateElement("Config");
				xmlDoc.AppendChild(rootConfig);
			
				// RFS Path
				XmlElement elem = xmlDoc.CreateElement("RFSPath");
				elem.AppendChild(xmlDoc.CreateTextNode(RFSToolPath));
				rootConfig.AppendChild(elem);
			
				// Temporary Directory
				elem = xmlDoc.CreateElement("TemporaryDirectory");
				elem.AppendChild(xmlDoc.CreateTextNode(TemporaryDirectory));
				rootConfig.AppendChild(elem);
			
				// Main form Properties
				XmlElement mainFormNode = xmlDoc.CreateElement("MainForm");
				mainFormNode.AppendChild(CreateElement(xmlDoc, "TreeWidth", MainForm_TreeView_Width.ToString()));
				XmlElement listViewNode = xmlDoc.CreateElement("ListView");
				listViewNode.AppendChild(CreateElement(xmlDoc, "Column_LastModified_Width", MainForm_ListView_LastMod_Width.ToString()));
				listViewNode.AppendChild(CreateElement(xmlDoc, "Column_Permissions_Width", MainForm_ListView_Perm_Width.ToString()));
				listViewNode.AppendChild(CreateElement(xmlDoc, "Column_Name_Width", MainForm_ListView_Name_Width.ToString()));
				listViewNode.AppendChild(CreateElement(xmlDoc, "Column_Size_Width", MainForm_ListView_Size_Width.ToString()));
				listViewNode.AppendChild(CreateElement(xmlDoc, "Column_View_Width", MainForm_ListView_View));
				mainFormNode.AppendChild(listViewNode);
			
				XmlNode root = xmlDoc.DocumentElement;
				rootConfig.AppendChild(mainFormNode);
				xmlDoc.Save(configFilename);
			} catch (Exception ex) {
				System.Windows.Forms.MessageBox.Show(ex.ToString());
			}
		}
		public XmlElement CreateElement(XmlDocument xmlDoc, string name, string nodeValue) {
			XmlElement elem = xmlDoc.CreateElement(name);
			elem.AppendChild(xmlDoc.CreateTextNode(nodeValue));
			return elem; 
		}
	}
}
